<?php

class GraphData
{

    private $dataSource;
    private $dataFile;
    private $graphType;
    private $planes;

    public function __construct($dataSource, $planes)
    {
        $this->dataSource = $dataSource;
        $this->planes = $planes;
    }


    public function getDataFile()
    {
        switch ($this->dataSource) {
            case "sealevelaccel":
                $this->dataFile = "sealevelaccel.csv";
                $this->graphType = "bar";
                break;

            case "lethality":
                $this->dataFile = "lethality.csv";
                $this->graphType = "stacked_column";
                break;

            case "speed":
                $this->dataFile = "speed_mil.csv";
                $this->graphType = "line";
                break;

            case "climb":
                $this->dataFile = "climb_mil.csv";
                $this->graphType = "line";
                break;

            case "speedwep":
                $this->dataFile = "speed_wep.csv";
                $this->graphType = "line";
                break;

            case "climbwep":
                $this->dataFile = "climb_wep.csv";
                $this->graphType = "line";
                break;

            case "accel3":
                $this->dataFile = "accel3.csv";
                $this->graphType = "stacked_column";
                break;

            case "turnradius":
                $this->dataFile = "turnradius.csv";
                $this->graphType = "double_bar";
                break;

            case "firetime":
                $this->dataFile = "firetime.csv";
                $this->graphType = "double_bar";
                break;

            case "topend":
                $this->dataFile = "topend.csv";
                $this->graphType = "double_bar";
                break;

            case "topendaccel":
                $this->dataFile = "topendaccel.csv";
                $this->graphType = "stacked_column";
                break;

            default:
                break;
        }

        return "data/" . $this->dataFile;
    }

    //get raw data of data
    public function getRawData()
    {
        $return = [];

        $file = $this->getDataFile();
        $head = "false";
        $delim = ",";
        $len = 1000;

        $handle = fopen($file, "r");
        if ($head) {
            $header = fgetcsv($handle, $len, $delim);
            // echo $header[0];
        }
        while (($data = fgetcsv($handle, $len, $delim)) !== FALSE) {
            if ($head and isset($header)) {
                foreach ($header as $key => $heading) {
                    $row[$heading] = (isset($data[$key])) ? $data[$key] : '';
                }
                $return[] = $row;
            } else {
                $return[] = $data;
            }
        }
        fclose($handle);
        return $return;
    }


    //get raw data of data
    public function getHeader()
    {
        $return = [];

        $file = $this->getDataFile();
        $head = "false";
        $delim = ",";
        $len = 1000;

        $handle = fopen($file, "r");
        $header = fgetcsv($handle, $len, $delim);
        fclose($handle);
        return $header;
    }


    public function formattedData()
    {
        $rawData = $this->getRawData();
        // return $rawData;
        if ($this->graphType == "line") {
            return $this->getLineGraphData($rawData);
        } else {
            return $this->getBarGraphData($rawData);
        }
    }


    public function getPlane($plane)
    {
        $planes = array(
            'temp' => 'Tempest',
            'spit16' => 'Spitfire Mk.XVI',
            'spit14' => 'Spitfire Mk.XIV',
            '109k4' => 'Bf 109K-4',
            'la7' => 'La-7',
            '109g14' => 'Bf 109G-14',
            'spit8' => 'Spitfire Mk.VIII',
            'la5fn' => 'La-5FN',
            'f4u4' => 'F4U-4',
            '190d9' => 'Fw 190D-9',
            'ki84' => 'Ki-84',
            '109g2' => 'Bf 109G-2',
            'n1k2j' => 'N1K2-J',
            '109f4' => 'Bf 109F-4',
            '109g6' => 'Bf 109G-6',
            'p38j' => 'P-38J',
            'yak9u' => 'Yak-9U',
            'p38l' => 'P-38L',
            'c205' => 'C.205', 'ita',
            'ta152h' => 'Ta 152H',
            'typh' => 'Typhoon',
            '190a5' => 'Fw 190A-5',
            'f4u1d' => 'F4U-1D',
            'spit9' => 'Spitfire Mk.IX',
            '190a8' => 'Fw 190A-8',
            'p51d' => 'P-51D',
            'f4u1c' => 'F4U-1C',
            '190f8' => 'Fw 190F-8',
            'f6f5' => 'F6F-5',
            'p47n' => 'P-47N', 'usa',
            'p47d40' => 'P-47D-40',
            'p51b' => 'P-51B',
            'c202' => 'C.202', 'ita',
            'mosq' => 'Mosquito',
            'p38g' => 'P-38G',
            'p47d25' => 'P-47D-25',
            'f4u1' => 'F4U-1',
            'p47d11' => 'P-47D-11',
            'spit1' => 'Spitfire Mk.I',
            'spit5' => 'Spitfire Mk.V',
            'yak9t' => 'Yak-9T',
            'a6m5' => 'A6M5',
            'fm2' => 'FM-2',
            'ki61' => 'Ki-61',
            '110g2' => 'Bf 110G-2',
            'seaf2' => 'Seafire Mk.II',
            'p40e' => 'P-40E',
            '109e4' => 'Bf 109E-4',
            'hurri2c' => 'Hurricane Mk.IIC',
            '110c4b' => 'Bf 110C-4b',
            'f4f4' => 'F4F-4',
            'a6m2' => 'A6M2',
            'hurri2d' => 'Hurricane Mk.IID',
            'hurri1' => 'Hurricane Mk.I',
            'p40b' => 'P-40B'
        );

        return $planes[$plane];
    }


    function random_color_part()
    {
        return str_pad(dechex(mt_rand(0, 255)), 2, '0', STR_PAD_LEFT);
    }

    function random_color($index)
    {
        $colors = ["#ff0030", "#0018ff", "#0fb30b", "#be8f44", "#f6ff00", "#c015da"];
        return $colors[$index];
        // return $this->random_color_part() . $this->random_color_part() . $this->random_color_part();
    }


    public function getGraphType()
    {
        if ($this->graphType == "line") {
            return "line";
        } else {
            return "bar";
        }
    }

    public function getOptions()
    {
        if ($this->graphType == "line") {
            return "{
                responsive:true,
                scales: {
                    y: {
                        beginAtZero: true,
                    }
                }
            }";
        } elseif ($this->graphType == "stacked_column") {
            return "{
                responsive:true,
                scales: {
                    y: {
                        beginAtZero: true,
                        stacked: true
                    },
                    x: {
                        beginAtZero: true,
                        stacked: true
                    }
                }
            }";
        } else {
            return "{
                responsive:true,
                scales: {
                    y: {
                        beginAtZero: true,

                    },
                    x: {
                        beginAtZero: true,
                    }
                }
            }";
        }
    }

    public function getLineGraphData($rawData)
    {
        $dataset = [];
        $labels = [];
        $count = 0;

        foreach ($rawData as $data) {

            $temp = [];
            $values = [];
            $curr_plane = "";
            $keys = [];

            foreach ($data as $key => $value) {

                if (strtolower($key) == "plane") {
                    $curr_plane = $value;
                } else {
                    $values[] = $value;
                    $keys[] = $key;
                }
            }

            $labels = $keys;
            $temp["label"] = $this->getPlane($curr_plane);

            $color = $this->random_color($count);

            $temp["backgroundColor"] = $color;
            $temp["borderColor"] = $color;
            $temp["data"] = $values;


            if (in_array(strtolower($curr_plane), $this->planes)) {
                $dataset["datasets"][] = $temp;
                $count++;
            }
        }


        $dataset["labels"] = $labels;


        return $dataset;
    }

    public function getBarGraphData($rawData)
    {
        $dataset = [];
        $labels = [];
        $tempData = [];
        $count = 0;

        foreach ($rawData as $data) {

            $plane = $data["Plane"];

            foreach ($data as $key => $value) {
                if (strtolower($key) != "plane") {
                    if (in_array(strtolower($plane), $this->planes)) {
                        $tempData[$plane][$key] = $value;
                    }
                }
            }
        }

        $temp = [];
        foreach ($tempData as $key => $value) {
            $labels[] = $this->getPlane($key);

            foreach ($value as $k => $val) {

                if ($this->dataSource == "lethality") {

                    if (strtolower($k) == "hammer/rtkh") {
                        $temp[$k][] = $val;
                    }
                } else {
                    $temp[$k][] = $val;
                }
            }
        }


        foreach ($temp as $field => $values) {
            $t = [];
            $t["label"] = $field;

            $color = $this->random_color($count);

            $t["backgroundColor"] = $color;
            $t["borderColor"] = $color;
            $t["data"] = $values;

            $dataset["datasets"][] = $t;
            $count++;
        }

        $dataset["labels"] = $labels;

        return $dataset;
    }
}
